package com.incubator.game.robot;

import com.incubator.common.log4j.Log;
import com.incubator.game.data.po.PlayerInfoPO;
import com.incubator.game.player.GPlayer;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.util.GDUtils;
import com.incubator.game.util.RobotUtil;
import com.incubator.message.proto.CommonProto;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 机器人管理类
 *
 * @author johnc
 */
public class RobotService {

    private static RobotService instance;
    /** 机器人对象池 **/
    public RobotPool robotPool;
    /** 使用中的机器人集合 **/
    public Map<String, GPlayer> robotPlayers = new ConcurrentHashMap<>();

    public synchronized static RobotService getInstance() {
        if (instance == null) {
            instance = new RobotService();
        }
        return instance;
    }

    /**
     * 初始化
     */
    public void init() {
        Log.info("初始化机器人服务...");
        this.robotPool = new RobotPool();
    }

    /**
     * 机器人加入房间
     *
     * @param room
     */
    public void joinRoom(Room room) {
        if (room == null) {
            return;
        }
        int robotNum = room.data.maxNum - room.actors.size();
        if (robotNum > 0) {
            List<Player> robotList = RobotUtil.createRobot(robotNum);
            if (!robotList.isEmpty()) {
                for (Player robot : robotList) {
                    // 机器人加入房间
                    room.joinRoom(robot);
                    // 机器人准备
                    robot.data.state = 1;
                    // 广播房间其他3个玩家
                    for (Player tmPlayer : room.actors.values()) {
                        if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), robot.getId())) {
                            Map<String, Object> data = room.roomToMessage(tmPlayer, null);
                            tmPlayer.receive(CommonProto.Cmd.ReadyStateUpdate_VALUE, data);
                        }
                    }
                    // 添加机器人集合
                    this.robotPlayers.put(robot.getId(), robot);
                }
            }
        }
    }

    /**
     * 机器人出牌
     *
     * @param room
     * @param robot
     */
    public void robotDisCard(Room room, Player robot) {
        if (room == null || robot == null) {
            return;
        }
        Log.info("机器人出牌...机器人ID:{}", robot.getId());
        int card = robot.data.remainCards[robot.data.remainCards.length - 1];
        room.playerDisCard(robot, new int[]{card}, GDUtils.CardType.SINGLE);
    }

    /**
     * 机器人操作过
     *
     * @param room
     * @param robot
     */
    public void robotPass(Room room, Player robot) {
        if (room == null || robot == null) {
            return;
        }
        Log.info("机器人过...机器人ID:{}", robot.getId());
        room.playerPass(robot, true);
    }
}
